<?php
/*
 * tb_exercise.php - Vips plugin for Stud.IP
 * Copyright (c) 2003-2005  Erik Schmitt, Philipp Hgelmeyer
 * Copyright (c) 2005-2006  Christa Deiwiks
 * Copyright (c) 2006-2007  Elmar Ludwig, Martin Schrder
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 */

require_once 'exercises/exercise.php';

/**
 * XXX detailed description
 *
 * @author    XXX
 * @copyright XXX
 * @abstract
 */
class tb_exercise extends Exercise {
    
//      #var $g_numberOfAnswers     = 8;
//      var $g_ExerciseName        = "Text Box";
//      var $g_ExerciseDescription = "Freie Antwortbox";
    
    function tb_exercise() {
//              //$this->g_ExerciseName        = "Freie Antwort";
//              //$this->g_ExerciseDescription = "Freie Antwort";
//              #$this->g_numberOfAnswers   = 8;
//              $this->g_Action                         = getExternalParam('action');
//              $this->g_OpenObject = 'openobject_' 
//                                                      . strtolower($this->Action);
    }
    
    
    
    /**
     *    Evaluates a solution.
     *
     *    @abstract
     *    @access public
     *    @returns int (from 0 to 100 ) as an % value
     */
    
    function evaluate ($exercise, $solution)
    {
        $dom_xml = $this->getDomXML($exercise);
        $answers = $dom_xml->get_elements_by_tagname("Answer");
        $node = $answers[0];
        $musterLoesung  = utf8_decode($node->get_content());

        $answerDefault  = $this->getTagsFromXML($exercise, "AnswerDefault");
        $answerDefault  = $answerDefault[0];
        
        $answer_distance = $this->getTagsFromXML($exercise, "AnswerDistance");
        $answer_distance = $answer_distance[0];
        
        $studentSolution = $this->getTagsFromXML($solution, "answer");
        $studentSolution = $studentSolution[0];
        
        $similarity = 0; // between 0 and 1

        $answerDefault   = normalizeText($answerDefault);
        $studentSolution = normalizeText($studentSolution);
        $musterLoesung   = normalizeText($musterLoesung);

        if ($studentSolution == '' || $studentSolution == $answerDefault)
        {
            $m_return[1] = 0;       // no points
            $m_return[2] = 1;
            return $m_return;
        }

        if ($musterLoesung == $studentSolution)
        {
            $similarity = 1;
        }
        else if ($answer_distance == 1)
        {
            // similar_text, returns the number of matching characters
            // and not levenshtein, returns the number of insert, replace
            // and delete operations needed to transform str1 into str2.

            $divisor1 = max(strlen($answerDefault), strlen($studentSolution));
            $divisor2 = max(strlen($answerDefault), strlen($musterLoesung));
            $divisor3 = max(strlen($studentSolution), strlen($musterLoesung));

            $distance1 = similar_text($answerDefault, $studentSolution) / $divisor1;
            $distance2 = similar_text($answerDefault, $musterLoesung) / $divisor2;
            $distance3 = similar_text($studentSolution, $musterLoesung) / $divisor3;

            if ($answerDefault == $musterLoesung)
                $similarity = $distance3;
            else
                $similarity = min((1-$distance1) / (1-$distance2) * $distance3, 0.9);
        }
        else if ($answer_distance == 2)
        {
            // Soundex-values: four character string
            // [0] first letter of encoded expression
            // [1]-[3] digits, somehow representing the sound value

            $levenshtein = levenshtein(soundex($musterLoesung), soundex($studentSolution));         

            if (soundex($musterLoesung) == soundex($studentSolution))
                $similarity = 0.8;
            else if ($levenshtein == 1)
                $similarity = 0.6;
            else if ($levenshtein == 2)
                $similarity = 0.4;
            else if ($levenshtein == 3)
                $similarity = 0.2;
            else  // $levenshtein == 4
                $similarity = 0;
        }

        $m_return[1] = $similarity * 100;       // percent
        $m_return[2] = $similarity == 1;
        return $m_return;
    }
    
    
    
    /**
     *    This function gens a Solution, that is stored to the database.
     *    solution_id.
     *
     *    @access public
     *    @returns XML contains the solution coded in an XML-string
     */
    function genSolution($exercise_id, $p_number_of_answers) {
        
        $i   = 0;
        $msg = '<solution id="'.$exercise_id.'">';
        
        // normally, only one solution can be entered by the student, anyway,
        // it could be usefull later to have more than one solution
        //#cdeiwiks: then I'll change it back...
        
        $msg .= '<answer id="0">';
        $msg .= '<body>'.htmlspecialchars(getExternalParam("answer_0")).'</body>';
        $msg .= '</answer>';
        
        $msg.= '</solution>';
        return ($msg);
    }
    
    
    
    /**
     *    Generate the XML-input for the database for an exercise.
     *
     *    @abstract
     *    @access public
     *    @returns bool
     */
    
    function getExerciseConstruct() {
        
        /**********************************************************************/
        /* Get data
        /*
        /* exercise_type = name of this class
        /**********************************************************************/
        
        $m_exercise_language = "de";
        $m_exercise_type     = getExternalParam("exercise_type");
        $m_exercise_question = getExternalParam("exercise_question");
        $m_exercise_name     = getExternalParam("exercise_name");
        $m_exercise_hint     = getExternalParam("exercise_hint");
        $m_exercise_comment  = getExternalParam("exercise_comment");
        
        $m_exercise_pic_url  = getExternalParam("exercise_pic_url");
        $m_provides          = getExternalParam("provides");
        $m_depends           = getExternalParam("depends");
        $m_optional_depends  = getExternalParam("optional_depends");
        
        $m_answer_distance   = getExternalParam("answer_distance");             // method of distance calculation
        $m_exercise_answer   = getExternalParam("answer_0");
        $m_answer_default    = getExternalParam("answer_default");
        
        $counter             = 1;
        
        /**********************************************************************/
        /* Construct XML
        /**********************************************************************/
        
        $xmlop  = "<TestItem ID=\"" . $m_exercise_type . "\" Type=\"".$this->g_ExerciseName."\" Language=\"" . $m_exercise_language . "\">";
        $xmlop .= "<Name>".htmlspecialchars($m_exercise_name)."</Name>";
        $xmlop .= "<Hint>".htmlspecialchars($m_exercise_hint)."</Hint>";
        $xmlop .= "<CommentField Visible=\"".$m_exercise_comment."\"></CommentField>";
        $xmlop .= "<PictureURL>".htmlspecialchars($m_exercise_pic_url)."</PictureURL>";
        $xmlop .= "<Provides>".htmlspecialchars($m_provides)."</Provides>";
        $xmlop .= "<Depends>".htmlspecialchars($m_depends)."</Depends>";
        $xmlop .= "<OptionalDepends>".htmlspecialchars($m_optional_depends)."</OptionalDepends>";
        $xmlop .= "<AnswerDistance>".htmlspecialchars($m_answer_distance)."</AnswerDistance>";
        $xmlop .= "<AnswerDefault>".htmlspecialchars($m_answer_default)."</AnswerDefault>";
        $xmlop .= "<Question>";
        $xmlop .= "<Paragraph>".htmlspecialchars($m_exercise_question)."</Paragraph>";
        $xmlop .= "</Question>";
        $xmlop .= "<Answer>".htmlspecialchars($m_exercise_answer)."</Answer>";
        
        $xmlop .= "</TestItem>";
        
        /**********************************************************************/
        /* Return XML
        /**********************************************************************/
        
        return ($xmlop);
    }

    /**
     * Export this exercise to plain text format.
     */
    function exportText ($exercise_xml)
    {
        $result = parent::exportText($exercise_xml, 'Offene Frage');

        $dom_xml = $this->getDomXML($exercise_xml);
        $answers = $dom_xml->get_elements_by_tagname('Answer');

        $answer_text     = utf8_decode($answers[0]->get_content());
        $answer_default  = $this->getTagsFromXML($exercise_xml, 'AnswerDefault');
        $answer_default  = $answer_default[0];
        $answer_distance = $this->getTagsFromXML($exercise_xml, 'AnswerDistance');
        $answer_distance = $answer_distance[0];

        if ($answer_distance == 1)
            $result .= "Abgleich: Levenshtein\n";

        if ($answer_default != '')
        {
            $result .= "Vorgabe:\n";
            $result .= $answer_default."\n";
            $result .= "\\Vorgabe\n";
        }

        if ($answer_text != '')
        {
            $result .= "Antwort:\n";
            $result .= $answer_text."\n";
            $result .= "\\Antwort\n";
        }

        return $result;
    }
}
?>
